@@ -1,15 +1,28 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * E-table-sorted.c: Implements a table that sorts another table
+ * e-table-sorted.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * Author:
- *   Miguel de Icaza (miguel@gnu.org)
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
  *
- * (C) 1999 Ximian, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
 #include <config.h>
 #include <stdlib.h>
-#include <gtk/gtksignal.h>
 #include <string.h>
 #include "gal/util/e-util.h"
 #include "e-table-sorted.h"
@@ -17,8 +30,6 @@
 
 #define d(x)
 
-#define PARENT_TYPE E_TABLE_SUBSET_TYPE
-
 #define INCREMENT_AMOUNT 100
 
 /* maximum insertions between an idle event that we will do without scheduling an idle sort */
@@ -35,35 +46,38 @@ static void ets_proxy_model_rows_inserted (ETableSubset *etss, ETableModel *sour
 static void ets_proxy_model_rows_deleted  (ETableSubset *etss, ETableModel *source, int row, int count);
 
 static void
-ets_destroy (GtkObject *object)
+ets_dispose (GObject *object)
 {
 	ETableSorted *ets = E_TABLE_SORTED (object);
 
-	if (ets->sort_idle_id) {
+	if (ets->sort_idle_id)
 		g_source_remove(ets->sort_idle_id);
-	}
-	if (ets->insert_idle_id) {
+	ets->sort_idle_id = 0;
+
+	if (ets->insert_idle_id)
 		g_source_remove(ets->insert_idle_id);
-	}
+	ets->insert_idle_id = 0;
 
 	if (ets->sort_info) {
-		gtk_signal_disconnect (GTK_OBJECT (ets->sort_info),
-				       ets->sort_info_changed_id);
-		gtk_object_unref(GTK_OBJECT(ets->sort_info));
+		g_signal_handler_disconnect (G_OBJECT (ets->sort_info),
+				             ets->sort_info_changed_id);
+		g_object_unref(ets->sort_info);
+		ets->sort_info = NULL;
 	}
 
 	if (ets->full_header)
-		gtk_object_unref(GTK_OBJECT(ets->full_header));
+		g_object_unref(ets->full_header);
+	ets->full_header = NULL;
 
-	GTK_OBJECT_CLASS (ets_parent_class)->destroy (object);
+	G_OBJECT_CLASS (ets_parent_class)->dispose (object);
 }
 
 static void
-ets_class_init (GtkObjectClass *object_class)
+ets_class_init (GObjectClass *object_class)
 {
 	ETableSubsetClass *etss_class = E_TABLE_SUBSET_CLASS(object_class);
 
-	ets_parent_class = gtk_type_class (PARENT_TYPE);
+	ets_parent_class = g_type_class_peek_parent (object_class);
 
 	etss_class->proxy_model_changed = ets_proxy_model_changed;
 	etss_class->proxy_model_row_changed = ets_proxy_model_row_changed;
@@ -71,7 +85,7 @@ ets_class_init (GtkObjectClass *object_class)
 	etss_class->proxy_model_rows_inserted = ets_proxy_model_rows_inserted;
 	etss_class->proxy_model_rows_deleted = ets_proxy_model_rows_deleted;
 
-	object_class->destroy = ets_destroy;
+	object_class->dispose = ets_dispose;
 }
 
 static void
@@ -86,16 +100,16 @@ ets_init (ETableSorted *ets)
 	ets->insert_count = 0;
 }
 
-E_MAKE_TYPE(e_table_sorted, "ETableSorted", ETableSorted, ets_class_init, ets_init, PARENT_TYPE);
+E_MAKE_TYPE(e_table_sorted, "ETableSorted", ETableSorted, ets_class_init, ets_init, E_TABLE_SUBSET_TYPE)
 
 static gboolean
 ets_sort_idle(ETableSorted *ets)
 {
-	gtk_object_ref(GTK_OBJECT(ets));
+	g_object_ref(ets);
 	ets_sort(ets);
 	ets->sort_idle_id = 0;
 	ets->insert_count = 0;
-	gtk_object_unref(GTK_OBJECT(ets));
+	g_object_unref(ets);
 	return FALSE;
 }
 
@@ -110,23 +124,26 @@ ets_insert_idle(ETableSorted *ets)
 ETableModel *
 e_table_sorted_new (ETableModel *source, ETableHeader *full_header, ETableSortInfo *sort_info)
 {
-	ETableSorted *ets = gtk_type_new (E_TABLE_SORTED_TYPE);
+	ETableSorted *ets = g_object_new (E_TABLE_SORTED_TYPE, NULL);
 	ETableSubset *etss = E_TABLE_SUBSET (ets);
 
+	if (ets_parent_class->proxy_model_pre_change)
+		(ets_parent_class->proxy_model_pre_change) (etss, source);
+
 	if (e_table_subset_construct (etss, source, 0) == NULL){
-		gtk_object_unref (GTK_OBJECT (ets));
+		g_object_unref (ets);
 		return NULL;
 	}
 
 	ets->sort_info = sort_info;
-	gtk_object_ref(GTK_OBJECT(ets->sort_info));
+	g_object_ref(ets->sort_info);
 	ets->full_header = full_header;
-	gtk_object_ref(GTK_OBJECT(ets->full_header));
+	g_object_ref(ets->full_header);
 
 	ets_proxy_model_changed(etss, source);
 
-	ets->sort_info_changed_id = gtk_signal_connect (GTK_OBJECT (sort_info), "sort_info_changed",
-							 GTK_SIGNAL_FUNC (ets_sort_info_changed), ets);
+	ets->sort_info_changed_id = g_signal_connect (G_OBJECT (sort_info), "sort_info_changed",
+						      G_CALLBACK (ets_sort_info_changed), ets);
 
 	return E_TABLE_MODEL(ets);
 }
@@ -184,17 +201,27 @@ ets_proxy_model_rows_inserted (ETableSubset *etss, ETableModel *source, int row,
  	ETableModel *etm = E_TABLE_MODEL(etss);
 	ETableSorted *ets = E_TABLE_SORTED(etss);
 	int i;
+	gboolean full_change = FALSE;
 
-	e_table_model_pre_change (etm);
+	if (count == 0) {
+		e_table_model_no_change (etm);
+		return;
+	}
 
-	for (i = 0; i < etss->n_map; i++) {
-		if (etss->map_table[i] >= row)
-			etss->map_table[i] += count;
+	if (row != etss->n_map) {
+		full_change = TRUE;
+		for (i = 0; i < etss->n_map; i++) {
+			if (etss->map_table[i] >= row) {
+				etss->map_table[i] += count;
+			}
+		}
 	}
 
 	etss->map_table = g_realloc (etss->map_table, (etss->n_map + count) * sizeof(int));
 
 	for (; count > 0; count --) {
+		if (!full_change)
+			e_table_model_pre_change (etm);
 		i = etss->n_map;
 		if (ets->sort_idle_id == 0) {
 			/* this is to see if we're inserting a lot of things between idle loops.
@@ -214,11 +241,17 @@ ets_proxy_model_rows_inserted (ETableSubset *etss, ETableModel *source, int row,
 		}
 		etss->map_table[i] = row;
 		etss->n_map++;
+		if (!full_change) {
+			e_table_model_row_inserted (etm, i);
+		}
 
-		e_table_model_row_inserted (etm, i);
 		d(g_print("inserted row %d", row));
 		row++;
 	}
+	if (full_change)
+		e_table_model_changed (etm);
+	else
+		e_table_model_no_change (etm);
 	d(e_table_subset_print_debugging(etss));
 }
 
@@ -231,11 +264,12 @@ ets_proxy_model_rows_deleted (ETableSubset *etss, ETableModel *source, int row,
 	int j;
 
 	shift = row == etss->n_map - count;
-	
+
 	for (j = 0; j < count; j++) {
 		for (i = 0; i < etss->n_map; i++){
 			if (etss->map_table[i] == row + j) {
-				e_table_model_pre_change (etm);
+				if (shift)
+					e_table_model_pre_change (etm);
 				memmove (etss->map_table + i, etss->map_table + i + 1, (etss->n_map - i - 1) * sizeof(int));
 				etss->n_map --;
 				if (shift)
@@ -250,6 +284,8 @@ ets_proxy_model_rows_deleted (ETableSubset *etss, ETableModel *source, int row,
 		}
 
 		e_table_model_changed (etm);
+	} else {
+		e_table_model_no_change (etm);
 	}
 
 	d(g_print("deleted row %d count %d", row, count));
